package gssim.db.mysql;

import gssim.db.dao.data.Metaexperiment;
import gssim.db.find.Finder;
import gssim.db.utils.PreparedStatementAdapter;
import gssim.db.utils.ResultSetAdapter;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.LogFactory;

public class MetaexperimentDAO extends AbstractDAO implements
		gssim.db.dao.MetaexperimentDAO {

	protected MetaexperimentDAO(Connection conn) throws SQLException {
		super(conn);
		log = LogFactory.getLog(MetaexperimentDAO.class);
		setRelationName("metaexperiment");
	}

	@Override
	protected PreparedStatementAdapter createDeleteStmt() throws SQLException {
		String sql = "DELETE FROM metaexperiment WHERE id = ?";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createFindStmt() throws SQLException {
		String sql = "SELECT * FROM metaexperiment WHERE (name = ? OR ISNULL(?))";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createInsertStmt() throws SQLException {
		String sql = "INSERT INTO metaexperiment(name, description) VALUES(?, ?)";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql,
						Statement.RETURN_GENERATED_KEYS));

		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createSelectStmt() throws SQLException {
		String sql = "SELECT * FROM metaexperiment WHERE id = ?";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createUpdateStmt() throws SQLException {
		String sql = "UPDATE metaexperiment SET name = ?, description = ? WHERE id = ?";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	public boolean delete(Metaexperiment obj, boolean commit) throws SQLException {

		PreparedStatementAdapter stmt = getDeleteStmt();
		stmt.setInt(1, obj.getId());

		int result = stmt.executeUpdate();

		if (result == 1) {
			if (commit)
				commit();
			logSuccess("delete");
		} else
			logError("delete", result, obj);

		return (result == 1);
	}

	@Override
	public void fillObjectFromResultSet(Metaexperiment obj, ResultSet resultSet)
			throws SQLException {

		ResultSetAdapter result = (ResultSetAdapter) resultSet;

		Integer id = result.getIntObj("id");
		String name = result.getString("name");
		String description = result.getString("description");

		obj.setId(id);
		obj.setName(name);
		obj.setDescription(description);

	}

	@Override
	public int fillStatementWithObject(PreparedStatement pstmt, Metaexperiment obj,
			int offset) throws SQLException {

		PreparedStatementAdapter stmt = (PreparedStatementAdapter) pstmt;

		stmt.setString(offset + 1, obj.getName());
		stmt.setString(offset + 2, obj.getDescription());

		return offset + 2;

	}

	@Override
	public List<Metaexperiment> find(Finder finder) throws SQLException {

		PreparedStatementAdapter stmt = getFindStmt();
		List<Metaexperiment> list = new ArrayList<Metaexperiment>();

		stmt.setString(1, finder.getString("name"));
		stmt.setString(2, finder.getString("name"));

		ResultSetAdapter result = new ResultSetAdapter(stmt.executeQuery());

		while (result.next()) {
			Metaexperiment obj = new Metaexperiment();
			fillObjectFromResultSet(obj, result);
			list.add(obj);
		}

		return list;

	}

	@Override
	public int insert(Metaexperiment obj, boolean commit) throws SQLException {

		PreparedStatementAdapter stmt = getInsertStmt();

		fillStatementWithObject(stmt, obj, 0);

		int result = stmt.executeUpdate();

		if (result == 1) {
			ResultSet gk = stmt.getGeneratedKeys();
			Integer id = gk.next() ? gk.getInt(1) : -1;
			obj.setId(id);

			if (commit)
				commit();

			logSuccess("insert");

		} else
			logError("insert", result, obj);

		return result;
	}

	public int[] insertBatch(List<Metaexperiment> objList, boolean commit)
			throws SQLException {

		PreparedStatementAdapter stmt = getInsertStmt();

		for (int i = 0; i < objList.size(); i++) {
			fillStatementWithObject(stmt, objList.get(i), 0);
			stmt.addBatch();
		}

		int[] result = stmt.executeBatch();

		stmt.clearBatch();

		int j = 0;

		while (j < result.length)
			if (result[j] == 1)
				j++;

		if (j == result.length) {

			ResultSet gk = stmt.getGeneratedKeys();
			for (int i = 0; i < objList.size(); i++) {
				Integer id = gk.next() ? gk.getInt(1) : -1;
				objList.get(i).setId(id);
			}

			logSuccess("batch insert", j);

			if (commit)
				commit();

		} else
			logError("batch insert", j);

		return result;
	}

	@Override
	public Metaexperiment select(Metaexperiment obj) throws SQLException {

		PreparedStatementAdapter stmt = getSelectStmt();
		stmt.setInt(1, obj.getId());
		ResultSetAdapter result = new ResultSetAdapter(stmt.executeQuery());

		result.next();

		boolean first = result.isFirst();
		boolean last = result.isLast();

		if (first && last) {

			fillObjectFromResultSet(obj, result);
			return obj;
		}

		// no user found with specified id
		if (!first && !last) {
			return null;
		}

		// no idea how this may happen (only if id is not primary key)
		throw new SQLException("Select operation in relation "
				+ getRelationName() + " returned more than one record");

	}

	@Override
	public boolean update(Metaexperiment obj, boolean commit) throws SQLException {

		PreparedStatementAdapter stmt = getUpdateStmt();

		int offset = fillStatementWithObject(stmt, obj, 0);

		stmt.setInt(offset + 1, obj.getId());

		int result = stmt.executeUpdate();

		if (result == 1) {
			if (commit)
				commit();
			logSuccess("update");
		} else
			logError("update", result, obj);

		return (result == 1);
	}

}
