package gssim.db.mysql;

import gssim.db.dao.data.ResourceAllTimeline;
import gssim.db.find.Finder;
import gssim.db.utils.PreparedStatementAdapter;
import gssim.db.utils.ResultSetAdapter;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.LogFactory;

import com.mysql.jdbc.NotImplemented;

public class ResourceAllTimelineDAO extends AbstractDAO implements
		gssim.db.dao.ResourceAllTimelineDAO {

	protected ResourceAllTimelineDAO(Connection conn) throws SQLException {
		super(conn);
		log = LogFactory.getLog(ResourceAllTimelineDAO.class);
		setRelationName("resource_allocation_timeline");
	}

	@Override
	protected PreparedStatementAdapter createDeleteStmt() throws SQLException {
		return null;
	}

	@Override
	protected PreparedStatementAdapter createFindStmt() throws SQLException {
		String sql = "SELECT * FROM resource_allocation_timeline WHERE"
				+ " (fk_resource_id = ? OR ISNULL(?))"
				+ " AND (timestamp between ? AND ?)";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createInsertStmt() throws SQLException {
		String sql = "INSERT INTO resource_allocation_timeline VALUES(?, ?, ?)";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));

		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createSelectStmt() throws SQLException {
		String sql = "SELECT * FROM resource_allocation_timeline WHERE fk_resource_id = ? AND timestamp = ?";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createUpdateStmt() throws SQLException {
		return null;
	}

	protected PreparedStatementAdapter createSumStmt() throws SQLException {
		String sql = "SELECT timestamp, SUM(`usage`) as 'usage'" + 
					" FROM resource_allocation_timeline rat" +
					" JOIN resource r ON r.PK = rat.fk_resource_id" +
					" WHERE exp_id = ?" +
					" AND (timestamp between ? AND ?)" +
					" GROUP BY TIMESTAMP";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}
	
	@Override
	public boolean delete(ResourceAllTimeline obj, boolean commit)
			throws SQLException {
		throw new NotImplemented();
	}

	@Override
	public void fillObjectFromResultSet(ResourceAllTimeline obj,
			ResultSet result) throws SQLException {
		throw new NotImplemented();

	}

	@Override
	public int fillStatementWithObject(PreparedStatement stmt,
			ResourceAllTimeline obj, int offset) throws SQLException {
		throw new NotImplemented();
	}

	@Override
	public List<ResourceAllTimeline> find(Finder finder) throws SQLException {

		PreparedStatementAdapter stmt = getFindStmt();
		List<ResourceAllTimeline> list = new ArrayList<ResourceAllTimeline>();

		stmt.setInt(1, finder.getInteger("fkResourceId"));
		stmt.setInt(2, finder.getInteger("fkResourceId"));
		stmt.setTimestamp(3, finder.getMinTimestamp("timestamp"));
		stmt.setTimestamp(4, finder.getMaxTimestamp("timestamp"));

		ResultSetAdapter result = new ResultSetAdapter(stmt.executeQuery());

		while (result.next()) {
			ResourceAllTimeline obj = new ResourceAllTimeline();

			Integer fkResourceId = result.getIntObj("fk_resource_id");
			Timestamp timestamp = result.getTimestamp("timestamp");
			Float usage = result.getFloatObj("usage");

			obj.setFkResourceId(fkResourceId);
			obj.setTimestamp(timestamp);
			obj.setUsage(usage);

			list.add(obj);
		}

		return list;
	}

	@Override
	public int insert(ResourceAllTimeline obj, boolean commit)
			throws SQLException {

		PreparedStatementAdapter stmt = getInsertStmt();

		stmt.setInt(1, obj.getFkResourceId());
		stmt.setTimestamp(2, obj.getTimestamp());
		stmt.setFloat(3, obj.getUsage());

		int result = stmt.executeUpdate();

		if (result == 1) {
			if (commit)
				commit();
			logSuccess("insert");
		} else
			logError("insert", result, obj);

		return result;
	}

	@Override
	public int[] insertBatch(List<ResourceAllTimeline> objList, boolean commit)
			throws SQLException {

		PreparedStatementAdapter stmt = getInsertStmt();

		for (int i = 0; i < objList.size(); i++) {
			stmt.setInt(1, objList.get(i).getFkResourceId());
			stmt.setTimestamp(2, objList.get(i).getTimestamp());
			stmt.setFloat(3, objList.get(i).getUsage());
			stmt.addBatch();
		}

		int[] result = stmt.executeBatch();

		stmt.clearBatch();

		int j = 0;

		while (j < result.length)
			if (result[j] == 1)
				j++;

		if (j == result.length) {
			if (commit)
				commit();

			logSuccess("batch insert", j);

		} else
			logError("batch insert", j);

		return result;
	}

	@Override
	public ResourceAllTimeline select(ResourceAllTimeline obj)
			throws SQLException {

		PreparedStatementAdapter stmt = getSelectStmt();
		stmt.setInt(1, obj.getFkResourceId());
		stmt.setTimestamp(2, obj.getTimestamp());
		ResultSetAdapter result = new ResultSetAdapter(stmt.executeQuery());

		result.next();
		boolean first = result.isFirst();
		boolean last = result.isLast();

		if (first && last) {

			obj.setUsage(result.getFloatObj("usage"));
			return obj;
		}

		// no reservation found with specified task_id and unit_id
		if (!first && !last) {
			return null;
		}

		// no idea how this may happen (only if id is not primary key)
		throw new SQLException("Select operation in relation "
				+ getRelationName() + " returned more than one record");
	}

	@Override
	public boolean update(ResourceAllTimeline obj, boolean commit)
			throws SQLException {
		throw new NotImplemented();
	}
	
	public List<ResourceAllTimeline> sum(Finder finder) throws SQLException {

		PreparedStatementAdapter stmt = createSumStmt();
		List<ResourceAllTimeline> list = new ArrayList<ResourceAllTimeline>();

		stmt.setInt(1, finder.getInteger("fkResourceId"));
		stmt.setTimestamp(2, finder.getMinTimestamp("timestamp"));
		stmt.setTimestamp(3, finder.getMaxTimestamp("timestamp"));
		ResultSetAdapter result = new ResultSetAdapter(stmt.executeQuery());

		while (result.next()) {
			ResourceAllTimeline obj = new ResourceAllTimeline();

			Timestamp timestamp = result.getTimestamp("timestamp");
			Float usage = result.getFloatObj("usage");

			obj.setTimestamp(timestamp);
			obj.setUsage(usage);

			list.add(obj);
		}

		return list;
	}

}